import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/models/settings.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/utils/date_time_utils.dart';
import 'package:diameter/utils/utils.dart';
import 'package:flutter/material.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/models/bolus.dart';
import 'package:diameter/models/bolus_profile.dart';

class BolusDetailScreen extends StatefulWidget {
  static const String routeName = '/bolus';

  final int bolusProfileId;
  final int id;
  final TimeOfDay? suggestedStartTime;
  final TimeOfDay? suggestedEndTime;

  const BolusDetailScreen(
      {Key? key,
      this.bolusProfileId = 0,
      this.id = 0,
      this.suggestedStartTime,
      this.suggestedEndTime})
      : super(key: key);

  @override
  _BolusDetailScreenState createState() => _BolusDetailScreenState();
}

class _BolusDetailScreenState extends State<BolusDetailScreen> {
  Bolus? _bolus;
  bool _isNew = true;
  bool _isSaving = false;

  final GlobalKey<FormState> _bolusForm = GlobalKey<FormState>();

  TimeOfDay _startTime = const TimeOfDay(hour: 0, minute: 0);
  TimeOfDay _endTime = const TimeOfDay(hour: 0, minute: 0);

  final _startTimeController = TextEditingController(text: '');
  final _endTimeController = TextEditingController(text: '');
  final _unitsController = TextEditingController(text: '');
  final _carbsController = TextEditingController(text: '');
  final _mgPerDlController = TextEditingController(text: '');
  final _mmolPerLController = TextEditingController(text: '');

  @override
  void initState() {
    super.initState();
    reload();

    if (widget.suggestedStartTime != null) {
      _startTime = widget.suggestedStartTime!;
    }
    if (widget.suggestedEndTime != null) {
      _endTime = widget.suggestedEndTime!;
    }

    if (_bolus != null) {
      _startTime = TimeOfDay.fromDateTime(_bolus!.startTime);
      _endTime = TimeOfDay.fromDateTime(_bolus!.endTime);

      _unitsController.text = _bolus!.units.toString();
      _carbsController.text = _bolus!.carbs.toString();
      _mgPerDlController.text = _bolus!.mgPerDl.toString();
      _mmolPerLController.text = _bolus!.mmolPerL.toString();
    }

    updateStartTime();
    updateEndTime();
  }

  void reload() {
    if (widget.id != 0) {
      setState(() {
        _bolus = Bolus.get(widget.id);
      });
    }
    _isNew = _bolus == null;
  }

  void updateStartTime() {
    _startTimeController.text = DateTimeUtils.displayTimeOfDay(_startTime);
  }

  void updateEndTime() {
    _endTimeController.text = DateTimeUtils.displayTimeOfDay(_endTime);
  }

  Future<String?> validateTimePeriod() async {
    String? error;
    List<Bolus> bolusRates = Bolus.getAllForProfile(widget.bolusProfileId);
    
    // check for duplicates
    if (bolusRates
        .where((other) =>
            widget.id != other.id &&
            _startTime.hour == other.startTime.hour &&
            _startTime.minute == other.startTime.minute)
        .isNotEmpty) {
      error = 'There\'s already a rate with this start time.';
    }

    if (bolusRates
        .where((other) =>
            (widget.id != other.id) &&
            DateTimeUtils.convertTimeOfDayToDateTime(_startTime)
                .isBefore(other.startTime) &&
            DateTimeUtils.convertTimeOfDayToDateTime(_endTime)
                .isAfter(other.startTime))
        .isNotEmpty) {
      error = 'This rate\'s time period overlaps with another one.';
    }

    return error == null
        ? null
        : showDialog(
            context: context,
            builder: (BuildContext context) {
              return AlertDialog(
                content: Text(error!),
                actions: <Widget>[
                  TextButton(
                    onPressed: () => Navigator.pop(context, 'CANCEL'),
                    child: const Text('GO BACK TO EDITING'),
                  ),
                  ElevatedButton(
                    onPressed: () => Navigator.pop(context, 'CONFIRM'),
                    child: const Text('SAVE AS IS'),
                  ),
                ],
              );
            });
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });

    if (_bolusForm.currentState!.validate()) {
      await validateTimePeriod().then((value) async {
        if (value != 'CANCEL') {
          Bolus bolus = Bolus(
            id: widget.id,
            startTime: DateTimeUtils.convertTimeOfDayToDateTime(_startTime),
            endTime: DateTimeUtils.convertTimeOfDayToDateTime(_endTime),
            units: double.tryParse(_unitsController.text) ?? 0,
            carbs: double.tryParse(_carbsController.text) ?? 0,
            mgPerDl: int.tryParse(_mgPerDlController.text),
            mmolPerL: double.parse(_mmolPerLController.text),
          );
          bolus.bolusProfile.targetId = widget.bolusProfileId;
          Bolus.put(bolus);
          Navigator.pop(context, '${_isNew ? 'New' : ''} Bolus Rate saved');
        }
      });
    }

    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    if (Settings.get().showConfirmationDialogOnCancel &&
        ((_isNew &&
                (_startTime.hour != (widget.suggestedStartTime?.hour ?? 0) ||
                    _endTime.hour != (widget.suggestedEndTime?.hour ?? 0) ||
                    _startTime.minute !=
                        (widget.suggestedStartTime?.minute ?? 0) ||
                    _endTime.minute != (widget.suggestedEndTime?.minute ?? 0) ||
                    (double.tryParse(_unitsController.text) ?? 0) != 0.0 ||
                    (double.tryParse(_carbsController.text) ?? 0) != 0.0 ||
                    (int.tryParse(_mgPerDlController.text) ?? 0) != 0 ||
                    (double.tryParse(_mmolPerLController.text) ?? 0) != 0.0)) ||
            (!_isNew &&
                (TimeOfDay.fromDateTime(_bolus!.startTime) != _startTime ||
                    TimeOfDay.fromDateTime(_bolus!.endTime) != _endTime ||
                    (double.tryParse(_unitsController.text) ?? 0) !=
                        _bolus!.units ||
                    (double.tryParse(_carbsController.text) ?? 0) !=
                        _bolus!.carbs ||
                    (double.tryParse(_mgPerDlController.text) ?? 0) !=
                        _bolus!.mgPerDl ||
                    (double.tryParse(_mmolPerLController.text) ?? 0) !=
                        _bolus!.mmolPerL)))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: _isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  void convertBetweenMgPerDlAndMmolPerL({GlucoseMeasurement? calculateFrom}) {
    int? mgPerDl;
    double? mmolPerL;

    if (calculateFrom != GlucoseMeasurement.mmolPerL &&
        _mgPerDlController.text != '') {
      mgPerDl = int.tryParse(_mgPerDlController.text);
    }
    if (calculateFrom != GlucoseMeasurement.mgPerDl &&
        _mmolPerLController.text != '') {
      mmolPerL = double.tryParse(_mmolPerLController.text);
    }

    if (mgPerDl != null && mmolPerL == null) {
      setState(() {
        _mmolPerLController.text =
            Utils.convertMgPerDlToMmolPerL(mgPerDl!).toString();
      });
    }
    if (mmolPerL != null && mgPerDl == null) {
      setState(() {
        _mgPerDlController.text =
            Utils.convertMmolPerLToMgPerDl(mmolPerL!).toString();
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(
            '${_isNew ? 'New' : 'Edit'} Bolus Rate for ${BolusProfile.get(widget.bolusProfileId)?.name}'),
      ),
      drawer: const Navigation(currentLocation: BolusDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          children: [
            FormWrapper(
              formState: _bolusForm,
              fields: [
                Row(
                  children: [
                    Expanded(
                      child: Padding(
                        padding: const EdgeInsets.only(right: 5),
                        child: TimeOfDayFormField(
                          label: 'Start Time',
                          controller: _startTimeController,
                          time: _startTime,
                          onChanged: (newStartTime) {
                            if (newStartTime != null) {
                              setState(() {
                                _startTime = newStartTime;
                              });
                              updateStartTime();
                            }
                          },
                        ),
                      ),
                    ),
                    Expanded(
                      child: Padding(
                        padding: const EdgeInsets.only(left: 5),
                        child: TimeOfDayFormField(
                          label: 'End Time',
                          controller: _endTimeController,
                          time: _endTime,
                          onChanged: (newEndTime) {
                            if (newEndTime != null) {
                              setState(() {
                                _endTime = newEndTime;
                              });
                              updateEndTime();
                            }
                          },
                        ),
                      ),
                    ),
                  ],
                ),
                TextFormField(
                  decoration: const InputDecoration(
                    labelText: 'Units',
                    suffixText: 'U',
                  ),
                  controller: _unitsController,
                  keyboardType:
                      const TextInputType.numberWithOptions(decimal: true),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty amount of units';
                    }
                    return null;
                  },
                ),
                TextFormField(
                  decoration: InputDecoration(
                    labelText: 'per carbs',
                    suffixText: nutritionMeasurementSuffixes[Settings.get().nutritionMeasurement.index],
                  ),
                  controller: _carbsController,
                  keyboardType:
                      const TextInputType.numberWithOptions(decimal: true),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'How many carbs does the rate make up for?';
                    }
                    return null;
                  },
                ),
                Row(
                  children: [
                    Settings.get().glucoseMeasurement == GlucoseMeasurement.mgPerDl ||
                            Settings.get().glucoseDisplayMode == GlucoseDisplayMode.both ||
                            Settings.get().glucoseDisplayMode ==
                                GlucoseDisplayMode.bothForDetail
                        ? Expanded(
                            child: TextFormField(
                              decoration: const InputDecoration(
                                labelText: 'per mg/dl',
                                suffixText: 'mg/dl',
                              ),
                              controller: _mgPerDlController,
                              onChanged: (_) =>
                                  convertBetweenMgPerDlAndMmolPerL(
                                      calculateFrom:
                                          GlucoseMeasurement.mgPerDl),
                              keyboardType:
                                  const TextInputType.numberWithOptions(),
                              validator: (value) {
                                if (value!.trim().isEmpty &&
                                    _mmolPerLController.text.trim().isEmpty) {
                                  return 'How many mg/dl does the rate make up for?';
                                }
                                return null;
                              },
                            ),
                          )
                        : Container(),
                    Settings.get().glucoseDisplayMode == GlucoseDisplayMode.both ||
                            Settings.get().glucoseDisplayMode ==
                                GlucoseDisplayMode.bothForDetail
                        ? IconButton(
                            onPressed: () => convertBetweenMgPerDlAndMmolPerL(
                                calculateFrom: GlucoseMeasurement.mmolPerL),
                            icon: const Icon(Icons.calculate),
                          )
                        : Container(),
                    Settings.get().glucoseMeasurement == GlucoseMeasurement.mmolPerL ||
                            Settings.get().glucoseDisplayMode == GlucoseDisplayMode.both ||
                            Settings.get().glucoseDisplayMode ==
                                GlucoseDisplayMode.bothForDetail
                        ? Expanded(
                            child: TextFormField(
                              decoration: const InputDecoration(
                                labelText: 'per mmol/l',
                                suffixText: 'mmol/l',
                              ),
                              controller: _mmolPerLController,
                              onChanged: (_) =>
                                  convertBetweenMgPerDlAndMmolPerL(
                                      calculateFrom:
                                          GlucoseMeasurement.mmolPerL),
                              keyboardType:
                                  const TextInputType.numberWithOptions(
                                      decimal: true),
                              validator: (value) {
                                if (value!.trim().isEmpty &&
                                    _mgPerDlController.text.trim().isEmpty) {
                                  return 'How many mmol/l does rhe rate make up for?';
                                }
                                return null;
                              },
                            ),
                          )
                        : Container(),
                    Settings.get().glucoseDisplayMode == GlucoseDisplayMode.both ||
                            Settings.get().glucoseDisplayMode ==
                                GlucoseDisplayMode.bothForDetail
                        ? IconButton(
                            onPressed: () => convertBetweenMgPerDlAndMmolPerL(
                                calculateFrom: GlucoseMeasurement.mgPerDl),
                            icon: const Icon(Icons.calculate),
                          )
                        : Container(),
                  ],
                ),
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
