import 'package:diameter/components/dialogs.dart';
import 'package:diameter/models/log_event.dart';
import 'package:diameter/models/settings.dart';
import 'package:diameter/screens/log/log_event/log_event_detail.dart';
import 'package:diameter/utils/date_time_utils.dart';
import 'package:flutter/material.dart';
import 'package:diameter/navigation.dart';

class LogEventListScreen extends StatefulWidget {
  static const String routeName = '/log-events';
  const LogEventListScreen({Key? key}) : super(key: key);

  @override
  _LogEventListScreenState createState() => _LogEventListScreenState();
}

class _LogEventListScreenState extends State<LogEventListScreen> {
  List<LogEvent> _activeEvents = [];
  late Map<DateTime, List<LogEvent>> _logEventDailyMap;

  @override
  void initState() {
    super.initState();
    reload();
  }

  void reload({String? message}) {
    setState(() {
      _activeEvents = LogEvent.getAllActiveForTime(DateTime.now());
      _logEventDailyMap = LogEvent.getDailyEntryMap();
    });

    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void handleAddNewEvent() async {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) {
          return const LogEventDetailScreen();
        },
      ),
    ).then((message) => reload(message: message));
  }

  void handleEditAction(LogEvent event) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => LogEventDetailScreen(
          id: event.id,
        ),
      ),
    ).then((message) => reload(message: message));
  }

  void onDelete(LogEvent logEvent) {
    LogEvent.remove(logEvent.id);
    reload(message: 'Event deleted');
  }

  void handleDeleteAction(LogEvent logEvent) async {
    if (Settings.get().showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(logEvent),
        message: 'Are you sure you want to delete this Event?',
      );
    } else {
      onDelete(logEvent);
    }
  }

  void onStop(LogEvent event) async {
    event.endTime = DateTime.now();
    LogEvent.put(event);
    reload(message: 'Event ended');
  }

  void handleStopAction(LogEvent event) async {
    if (Settings.get().showConfirmationDialogOnStopEvent) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onStop(event),
        message: 'Are you sure you want to end this Event?',
        confirmationLabel: 'END EVENT',
      );
    } else {
      onStop(event);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Log Events'),
        actions: <Widget>[
          IconButton(onPressed: reload, icon: const Icon(Icons.refresh))
        ],
      ),
      drawer: const Navigation(currentLocation: LogEventListScreen.routeName),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Expanded(
            child: _logEventDailyMap.isNotEmpty
                ? ListView.builder(
                    shrinkWrap: true,
                    padding: const EdgeInsets.all(10.0),
                    itemCount: _logEventDailyMap.length,
                    itemBuilder: (context, dateIndex) {
                      List<DateTime?> dateList = (_activeEvents.isNotEmpty
                              ? <DateTime?>[null]
                              : <DateTime?>[]) +
                          _logEventDailyMap.keys.toList();
                      final date = dateList[dateIndex];
                      final eventList = date != null
                          ? _logEventDailyMap[date]
                          : _activeEvents;
                      final tiles = <Widget>[];
                      if (eventList != null) {
                        for (LogEvent event in eventList) {
                          tiles.add(ListTile(
                                      onTap: () {
                                        handleEditAction(event);
                                      },
                                      title: Row(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.center,
                                        mainAxisSize: MainAxisSize.max,
                                        children: [
                                          Expanded(
                                              child: Text(date == null
                                                  ? DateTimeUtils
                                                      .displayDateTime(
                                                          event.time)
                                                  : DateTimeUtils.displayTime(
                                                      event.isEndEvent
                                                          ? event.endTime
                                                          : event.time))),
                                          const SizedBox(width: 24),
                                          Expanded(
                                            child: Text(event.title ??
                                                event.eventType.target?.value ??
                                                ''),
                                          ),
                                        ],
                                      ),
                                      trailing: Row(
                                        mainAxisSize: MainAxisSize.min,
                                        children: [
                                          event.hasEndTime &&
                                                  event.endTime == null
                                              ? IconButton(
                                                  icon: const Icon(
                                                    Icons.stop,
                                                    color: Colors.blue,
                                                  ),
                                                  onPressed: () =>
                                                      handleStopAction(event),
                                                )
                                              : const SizedBox(width: 50),
                                          IconButton(
                                            icon: const Icon(
                                              Icons.edit,
                                              color: Colors.blue,
                                            ),
                                            onPressed: () =>
                                                handleEditAction(event),
                                          ),
                                          IconButton(
                                            icon: const Icon(
                                              Icons.delete,
                                              color: Colors.blue,
                                            ),
                                            onPressed: () =>
                                                handleDeleteAction(event),
                                          ),
                                        ],
                                      ),
                                    ));
                        }
                      }
                      return eventList != null && eventList.isNotEmpty ? ListBody(
                        children: <Widget>[
                          Text(DateTimeUtils.displayDate(date,
                              fallback: 'Active Events'))
                        ] + tiles,
                          
                      ) : Container();
                    },
                  )
                : const Center(
                    child: Text('There are no Events!'),
                  ),
          ),
        ],
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: handleAddNewEvent,
        child: const Icon(Icons.add),
      ),
    );
  }
}
